/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.impl;

import it.tidalwave.bluebill.factsheet.GraphProvider;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.mobile.asset.SmartAssetManager;
import it.tidalwave.mobile.io.IoUtils;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.semantic.io.GraphDeserializer;
import it.tidalwave.semantic.io.MimeTypes;
import it.tidalwave.semantic.io.impl.DefaultGraphDeserializer;
import it.tidalwave.util.logging.Logger;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import javax.annotation.Nonnull;
import org.openrdf.model.Graph;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
// **********************************************************************************************************************/
public class DefaultGraphProvider implements GraphProvider
  {
    private static final String CLASS = DefaultGraphProvider.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private final Taxon taxon;

    public DefaultGraphProvider (final @Nonnull Taxon taxon)
      {
        this.taxon = taxon;
      }

    @Nonnull
    public Graph getGraph()
      throws IOException
      {
        ZipFile zipFile = null;
        InputStream is = null;

        try
          {
            final String resourceName = taxon.getId().stringValue().replaceAll(":", "_") + ".n3";
            logger.info(">>>> resourceName: %s", resourceName);

            final File file = Locator.find(SmartAssetManager.class).findAsset("factsheets.zip");
            logger.info(">>>> zip file: %s", file);
            zipFile = new ZipFile(file);
            final ZipEntry entry = zipFile.getEntry(resourceName);
            is = zipFile.getInputStream(entry);

            return loadGraph(is, MimeTypes.MIME_N3);
          }
        finally
          {
            IoUtils.safeClose(is);
            IoUtils.safeClose(zipFile);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected Graph loadGraph (final @Nonnull InputStream is, final @Nonnull String mimeType)
      throws IOException
      {
        try
          {
            final GraphDeserializer deserializer = new DefaultGraphDeserializer();
            return deserializer.read(is, mimeType);
          }
        catch (Exception e)
          {
            throw new IOException(e.toString()); // Java 5 / Android compatibility
          }
      }
  }
