/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.semantic.io;

import javax.annotation.Nonnull;
import java.util.List;
import it.tidalwave.util.As;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.semantic.io.spi.StatementUnmarshallerSupport;
import it.tidalwave.bluebill.factsheet.FactSheet;
import it.tidalwave.mobile.media.Media;
import org.openrdf.model.Statement;
import static it.tidalwave.semantic.vocabulary.FoafVocabulary.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class FactSheetUnmarshaller extends StatementUnmarshallerSupport
  {
    @Override @Nonnull
    public As unmarshal (final @Nonnull List<Statement> statements, final @Nonnull Context context)
      {
        final Id id = new Id(statements.get(0).getSubject().stringValue());
        FactSheet factSheet = new FactSheet(id);

        for (final Statement statement : context.findStatements(ID_FOAF_DEPICTS, id))
          {
            factSheet = factSheet.with(FOAF_IMAGE, (Media)context.find(statement.getSubject()));
          }

        try
          {
            final String string = findStatementWithPredicate(statements, FactSheet.ID_DESCRIPTION).getObject().stringValue();
            factSheet = factSheet.with(FactSheet.DESCRIPTION, string);
          }
        catch (NotFoundException e)
          {
            throw new RuntimeException(e);
            // ok. no description
          }

        return factSheet;
      }
  }
