/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.semantic.io;

import javax.annotation.Nonnull;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;
import org.openrdf.model.Statement;
import org.openrdf.model.Value;
import it.tidalwave.util.As;
import it.tidalwave.util.Id;
import it.tidalwave.util.Key;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.role.spi.DefaultDisplayable;
import it.tidalwave.semantic.GenericEntity;
import it.tidalwave.semantic.io.spi.StatementUnmarshallerSupport;
import static it.tidalwave.semantic.vocabulary.RdfsVocabulary.*;
import static it.tidalwave.semantic.vocabulary.FoafVocabulary.*;
import static it.tidalwave.semantic.vocabulary.DublinCoreVocabulary.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class GenericEntityUnmarshaller extends StatementUnmarshallerSupport
  {
    private static final List<Id> PREDICATES_REFERRING_IDS = Arrays.asList(ID_DC_IDENTIFIER);

    @Override @Nonnull
    public As unmarshal (final @Nonnull List<Statement> statements, final @Nonnull Context context)
      {
        final List<Object> capabilities = new ArrayList<Object>();
        capabilities.add(new Id(statements.get(0).getSubject().stringValue()));

        try
          {
            final String label = findStatementWithPredicate(statements, ID_RDFS_LABEL).getObject().stringValue();
            capabilities.add(new DefaultDisplayable(label, ""));
          }
        catch (NotFoundException e)
          {
            // ok
          }

        try
          {
            final String givenName = findStatementWithPredicate(statements, ID_FOAF_GIVEN_NAME).getObject().stringValue();
            capabilities.add(new DefaultDisplayable(givenName, ""));
          }
        catch (NotFoundException e)
          {
            // ok
          }

        GenericEntity entity = new GenericEntity(capabilities.toArray());

        for (final Statement statement : statements)
          {
            final Value object = statement.getObject();
            final Id predicate = new Id(statement.getPredicate().stringValue());
            final Key<Object> key = new Key<Object>(predicate);

//            if (statement.getPredicate().stringValue().equals(Vocabulary.DC_CREATOR.stringValue()))
//              {
//                media = media.with(key, context.find(object));
//              }
//            else if (object instanceof Literal)
              {
                if (PREDICATES_REFERRING_IDS.contains(predicate))
                  {
                    entity = entity.with(key, new Id(object.stringValue()));
                  }
                else
                  {
                    entity = entity.with(key, object.stringValue());
                  }
              }
          }

        return entity;
      }
  }
